﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

using Model;
using View;
using Microsoft.Xna.Framework;

namespace Controller
{
    /// <summary>
    /// Contains functions pertaining to the general game.
    /// </summary>

    /**
    * This class holds useable functions in the scope of the entire gameplay.
    */
    static class GameFunction
    {
        /**
        Returns whether or not the specified enemy unit is within attack range of the specified unit.
            \param graph Graph representing the current game map.
            \param unit Specified playable unit.
            \param enemyUnit Specified enemy unit.
        */
        public static bool isEnemyUnitInRange(Graph graph, Unit unit, Unit enemyUnit)
        {
            Node attackNode = graph.getNode(enemyUnit.Position.Item1, enemyUnit.Position.Item2);
            return isNodeAttackable(unit, unit.Position.Item1, unit.Position.Item2, attackNode);
        }

        /**
        Must be called upon the start of a new turn. Resets all unit buttons and game states.
            \param player Player of the new turn.
        */
        public static void startTurn(Player player)
        {
            // set all buttons to active
            foreach (Unit unit in player.getUnits())
            {
                Button[] buttons = unit.getButtons();
                for (int i = 0; i < buttons.Count(); i++)
                {
                    if (buttons[i].getButtonType() != ButtonType.Attack
                        || buttons[i].getButtonType() != ButtonType.AttackConfirm)
                    {
                        buttons[i].Active = true;
                    }
                }
            }

            // sets all game states
            GameState.selectedUnit = null;
            GameState.unitToAttack = null;
            GameState.dropDownMenuOpen = false;
            GameState.attackConfirmOpen = false;
            GameState.beforeMove = true;
        }

        /**
        Returns whether or not the specified unit can perform actions.
            \param unit Specified unit.
        */
        public static bool hasUnitFinishedActions(Unit unit)
        {
            if (!unit.isButtonActive(ButtonType.AttackConfirm))
            {
                return true;
            }
            if (!unit.isButtonActive(ButtonType.Wait))
            {
                return true;
            }
            return false;
        }

        /**
        Returns whether or not the current turn is over.
        */
        public static bool isTurnOver()
        {
            foreach (Unit unit in GameState.currentPlayer.getUnits())
            {
                if (!hasUnitFinishedActions(unit))
                {
                    return false;
                }
            }
            return true;
        }

        /**
        Returns whether or not the game is over, based off win conditions.
        */
        public static bool isGameOver(Player player1, Player player2)
        {
            // if player1 does not have anymore units, game is over
            if (player1.getNumOfUnits() <= 0)
            {
                return true;
            }

            // if player2 does not have anymore units, game is over
            if (player2.getNumOfUnits() <= 0)
            {
                return true;
            }

            return false;
        }

        /**
        Checks whether or not a specified node is attackable based on the unit's equipped weapon.
            \param unit Unit to be checked for attackable nodes.
            \param unitX The x coordinate of the unit (by nodes).
            \param unitY The y coordinate of the unit (by nodes).
            \param attackNode The specified attack node.
        */
        private static bool isNodeAttackable(Unit unit, int unitX, int unitY, Node attackNode)
        {
            int attackX = attackNode.getPositionX();    // coordinates of attack node
            int attackY = attackNode.getPositionY();
            int[] range = unit.equippedWeapon.range;    // range of unit's equipped weapon

            if (attackNode.isObstacle)
            {
                return false;   // if attack node is an obstacle, automatically non-attackable
            }

            // if attack node is within weapon's min and max attack range, then node is attackable
            if (Math.Abs(unitX - attackX) + Math.Abs(unitY - attackY) >= range[0] &&
                Math.Abs(unitX - attackX) + Math.Abs(unitY - attackY) <= range[1])
            {
                return true;
            }

            return false;
        }

        /**
        Returns a list of nodes that the unit can move onto.
            \param graph Graph representing the current game map.
            \param Specified unit.
        */
        public static LinkedList<Node> setMovableNodes(Graph graph, Unit unit)
        {
            LinkedList<Node> moveableNodes = new LinkedList<Node>();
            Node currentNode = graph.getNode(unit.Position);

            // iterate through all nodes in the graph
            for (int x = 0; x < graph.Width; x++)
            {
                for (int y = 0; y < graph.Height; y++)
                {
                    // if a path exists to that node, add it to list of moveable nodes
                    if (pathFinder(graph, unit, currentNode, graph.getNode(x, y)) != null && !graph.getNode(x, y).isOccupied())
                    {
                        moveableNodes.AddLast(graph.getNode(x, y));
                    }
                }
            }

            GameState.moveableNodes = moveableNodes;
            return moveableNodes;
        }

        /**
        Returns a list of nodes that the unit can perform an attack on. Assumes the unit has not yet moved.
            \param graph Graph representing the current game map.
            \param unit Specfied unit.
        */
        public static LinkedList<Node> getAttackableNodes(Graph graph, Unit unit)
        {
            LinkedList<Node> attackableNodes = new LinkedList<Node>();

            // determine attackable nodes for each moveable node
            foreach (Node moveableNode in GameState.moveableNodes)
            {
                // iterate through entire grid to determine attackable nodes for moveable node
                for (int x = 0; x < graph.Width; x++)
                {
                    for (int y = 0; y < graph.Height; y++)
                    {
                        if (isNodeAttackable(unit, moveableNode.getPositionX(), moveableNode.getPositionY(), graph.getNode(x, y)) &&
                            !attackableNodes.Contains(graph.getNode(x, y)))
                        {
                            // if node is attackable, add it to list of attackable nodes
                            attackableNodes.AddLast(graph.getNode(x, y));
                        }
                    }
                }
            }

            return attackableNodes;
        }

        /**
        Returns whether or not the specified enemy unit is within attack range of the specified unit. Assumes the unit has already moved.
            \param graph Graph representing the current game map.
            \param Specified unit.
        */
        public static LinkedList<Node> getAttackRangeAfterMoving(Graph graph, Unit unit)
        {
            LinkedList<Node> attackableNodes = new LinkedList<Node>();
            Tuple<int, int> currentPosition = unit.Position;
            for (int x = 0; x < graph.Width; x++)
            {
                for (int y = 0; y < graph.Height; y++)
                {
                    if (isNodeAttackable(unit, currentPosition.Item1, currentPosition.Item2, graph.getNode(x, y)))
                    {
                        // if node is attackable, add it to list of attackable nodes
                        attackableNodes.AddLast(graph.getNode(x, y));
                    }
                }
            }

            return attackableNodes;
        }

        /**
        Returns a list of nodes representing the path from start node to end node; if no path is valid, return null.
            \param graph Graph representing the current game map.
            \param unit Unit to move.
            \param start Start Node of the path.
            \param end End Node of the path
        */
        public static LinkedList<Node> pathFinder(Graph graph, Unit unit, Node start, Node end)
        {
            // TODO: (OPTIONAL, IGNORE FOR NOW) incorporate movability of nodes in the calculation

            // if end node is not movable, then there cannot be a path
            if (end.isObstacle || end.isOccupied())
            {
                return null;
            }

            // uses a variation of breadth-first search
            Queue<LinkedList<Node>> paths = new Queue<LinkedList<Node>>();  // queue of paths

            // sets up queue of paths
            LinkedList<Node> startPath = new LinkedList<Node>();
            startPath.AddFirst(start);      // adds the starting node to starting path
            paths.Enqueue(startPath);

            while (paths.Count != 0)    // while queue isn't empty
            {
                LinkedList<Node> path = paths.Dequeue();    // get the first path from the queue
                Node lastNode = path.Last();                // get the last node from the path
                if (lastNode == end)                        // if last node is desired end node, return path
                {
                    return path;
                }
                if (path.Count <= unit.getMovability())     // only consider paths within the unit's movability
                {
                    // add paths with adjacent nodes of last node to queue
                    foreach (Node adjacentNode in getAdjacentNodes(graph, lastNode))
                    {
                        if (!path.Contains(adjacentNode))   // if adjacent node doesn't already exist in the path
                        {
                            LinkedList<Node> newPath = new LinkedList<Node>();
                            // copies path to new path
                            foreach (Node node in path)
                            {
                                newPath.AddLast(node);
                            }
                            // add adjacent node to new path that will be enqueued
                            newPath.AddLast(adjacentNode);
                            paths.Enqueue(newPath);
                        }
                    }
                }
            }

            return null;
        }

        /**
        Returns moveable adjacent nodes of the specified node. Used for BFS path finding.
            \param graph Graph representing the current game map.
            \param Node to get moveable adjacent nodes of.
        */
        private static LinkedList<Node> getAdjacentNodes(Graph graph, Node node)
        {
            LinkedList<Node> adjacentNodes = new LinkedList<Node>();
            Node upNode = graph.getNode(node.getPositionX(), node.getPositionY() + 1);
            Node downNode = graph.getNode(node.getPositionX(), node.getPositionY() - 1);
            Node rightNode = graph.getNode(node.getPositionX() + 1, node.getPositionY());
            Node leftNode = graph.getNode(node.getPositionX() - 1, node.getPositionY());

            // up node
            if (!upNode.isOccupied() && !upNode.isObstacle)
            {
                adjacentNodes.AddLast(upNode);
            }

            // down node
            if (!downNode.isOccupied() && !downNode.isObstacle)
            {
                adjacentNodes.AddLast(downNode);
            }

            // left node
            if (!leftNode.isOccupied() && !leftNode.isObstacle)
            {
                adjacentNodes.AddLast(leftNode);
            }

            // right node
            if (!rightNode.isOccupied() && !rightNode.isObstacle)
            {
                adjacentNodes.AddLast(rightNode);
            }

            return adjacentNodes;
        }

        /**
        Removes the specified unit from the game.
            \param unit unit to remove from the game.
        */
        public static void removeUnit(Graph graph, Player player, Unit unit)
        {
            graph.getNode(unit.Position).unitOnNode = null;
            player.removeUnit(unit);
        }

        /**
        Deselects any selected unit.
        */
        public static void deselectUnit()
        {
            GameState.selectedUnit = null;
            GameState.selectedEnemyUnit = null;
            GameState.TurnState = TurnState.Wait;
            GameState.unitToAttack = null;
            GameState.attackConfirmOpen = false;
            GameState.dropDownMenuOpen = false;
            GameState.attackSelect = false;
        }

        /**
        If unit exists where user clicked (that belongs to player), return it; else, return null.
            \param clickedNode Node where user has clicked.
            \param positionClicked position (by node) of where the user has clicked.
            \param player Player that is currently moving.
        */
        public static Unit getUnitOnNodeClicked(Node clickedNode, Vector2 positionClicked, Player player)
        {
            for (int i = 0; i < player.getNumOfUnits(); i++)
            {
                Unit unit = player.getUnits().ElementAt(i);
                int unitX = unit.Position.Item1;
                int unitY = unit.Position.Item2;
                int clickedX = (int)Math.Floor(positionClicked.X / 32);
                int clickedY = (int)Math.Floor(positionClicked.Y / 32);
                if (unitX == clickedX && unitY == clickedY)
                {
                    return unit;
                }
            }
            return null;
        }

        /**
        Updates the unit's position to the clicked position.
            \param graph Graph representing the current game map.
            \param position Position (by node) to move the unit to.
            \param path Path to move the unit along.
        */
        public static void updateUnitPosition(Graph graph, Vector2 position, LinkedList<Node> path)
        {
            Unit unit = GameState.selectedUnit;
            GameState.isAnimating = true;

            // updates the unit's position to each node in the path (node by node)
            foreach (Node node in path)
            {
                Animation.animateUnitPosition(graph, unit, node);
                unit.Position = (new Tuple<int, int>(node.getPositionX(), node.getPositionY()));
                node.unitOnNode = (unit);
                GameState.dropDownMenuOpen = true;
            }

            GameState.isAnimating = false;
        }

        /**
        Ends the current player's turn and starts the enemy player's turn.
        */
        public static void endTurn()
        {
            Player tempPlayer = GameState.currentPlayer;
            GameState.currentPlayer = (GameState.enemyPlayer);
            GameState.enemyPlayer = (tempPlayer);
            startTurn(GameState.currentPlayer);
            GameState.endTurnButton = false;
            GameState.transitionTurn = true;
        }

        /**
        Updates the Model based upon button clicks.
            \param button Button that was clicked.
            \param graph Graph of the map.
        */
        // TODO: MOVE ANIMATION PARTS TO ANIMATION CLASS
        public static void buttonAction(Button button, Graph graph)
        {
            Unit unit = GameState.selectedUnit;
            Unit enemyUnit = GameState.unitToAttack;

            switch (button.getButtonType())
            {
                #region Attack Button
                // if attack clicked
                case ButtonType.Attack:
                    GameState.TurnState = TurnState.Attack;
                    GameState.dropDownMenuOpen = false; // close the dropdown menu when selecting who to attack
                    GameState.attackSelect = true;
                    break;
                #endregion

                #region Attack Confirm Button
                // if confirm attack clicked
                case ButtonType.AttackConfirm:
                    button.Active = false;
                    unit.getButtonType(ButtonType.Move).Active = false;     // move button is no longer active
                    unit.getButtonType(ButtonType.Attack).Active = false;   // attack button is no longer active

                    //Gets attack animation direction
                    #region Attack Animation Direction
                    Direction attackDir = Direction.Up;
                    Direction counterAttackDir = Direction.Up;
                    if (unit.Position.Item1 < enemyUnit.Position.Item1) // attack right
                    {
                        attackDir = Direction.Right;
                        counterAttackDir = Direction.Left;
                        unit.currentFrame = 7;
                        enemyUnit.currentFrame = 4;
                    }
                    else if (unit.Position.Item1 > enemyUnit.Position.Item1) // attack left
                    {
                        attackDir = Direction.Left;
                        counterAttackDir = Direction.Right;
                        unit.currentFrame = 4;
                        enemyUnit.currentFrame = 7;
                    }
                    else if (unit.Position.Item2 < enemyUnit.Position.Item2) // attack down
                    {
                        attackDir = Direction.Down;
                        counterAttackDir = Direction.Up;
                        unit.currentFrame = 1;
                        enemyUnit.currentFrame = 10;
                    }
                    else if (unit.Position.Item2 > enemyUnit.Position.Item2) // attack up
                    {
                        attackDir = Direction.Up;
                        counterAttackDir = Direction.Down;
                        unit.currentFrame = 10;
                        enemyUnit.currentFrame = 1;
                    }
                    #endregion

                    bool magicalAttack = isMagicalAttack(unit); // find what attack type unit is making
                    int damageDealt = DamageCalculations.finalDamage(unit, enemyUnit, magicalAttack); // gets damage dealt
                    GameState.lastAttackingUnit = unit;
                    GameState.lastDefendingUnit = enemyUnit;
                    GameState.CurrentPlayerDamageDealt = damageDealt.ToString();
                    enemyUnit.Hp = enemyUnit.Hp - damageDealt; // updates hp
                    GameState.currentPlayerDamagePopup = true;
                    Animation.attackAnimation(attackDir, unit);

                    // if enemy unit is still alive, perform a counter attack
                    if (enemyUnit.Alive && isEnemyUnitInRange(graph, enemyUnit, unit))
                    {
                        Thread.Sleep(750);
                        magicalAttack = isMagicalAttack(enemyUnit); // gets attack type 
                        int damageTaken = DamageCalculations.finalDamage(enemyUnit, unit, magicalAttack); // damage dealt
                        GameState.EnemyPlayerDamageDealt = damageTaken.ToString();
                        unit.Hp = unit.Hp - damageTaken; // update hp
                        GameState.enemyPlayerDamagePopup = true;
                        Animation.attackAnimation(counterAttackDir, enemyUnit);
                    }
                    deselectUnit();
                    break;
                #endregion

                #region Move Button
                // if Move is clicked
                case ButtonType.Move:
                    if (button.Active)          // if unit hasn't already moved
                    {
                        GameState.TurnState = TurnState.Move;
                        GameState.dropDownMenuOpen = false;   // close the dropdownmenu when selecting where to move
                    }
                    break;
                #endregion

                #region Item Button
                case ButtonType.Items:          // if item is clicked
                    GameState.TurnState = TurnState.Items;
                    GameState.inventoryOpen = true;
                    break;
                #endregion

                #region Wait Button
                // if wait is clicked
                case ButtonType.Wait:
                    GameState.TurnState = TurnState.Wait;
                    deselectUnit();
                    button.Active = false;
                    break;
                #endregion

                #region Inventory Buttons
                case ButtonType.Inventory1:
                    GameState.inventoryOpen = false;
                    unit.equippedWeapon = button.weapon;
                    break;
                case ButtonType.Inventory2:
                    GameState.inventoryOpen = false;
                    unit.equippedWeapon = button.weapon;
                    break;
                case ButtonType.Inventory3:
                    GameState.inventoryOpen = false;
                    unit.equippedWeapon = button.weapon;
                    break;
                case ButtonType.Inventory4:
                    GameState.inventoryOpen = false;
                    unit.equippedWeapon = button.weapon;
                    break;
                #endregion

                default:
                    break;
            }
        }

        /**
        Returns whether or not the unit performed a magical attack.
           \param unit Unit to check.
       */
        // TODO: change this function to factor the weapon type rather than unit type
        public static bool isMagicalAttack(Unit unit)
        {
            if ((unit.getClass() == UnitType.Warrior) || (unit.getClass() == UnitType.Archer))
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        /**
        Returns the menu button that was clicked; if no menu button was clicked, returns null.
            \param mouseCoordinates Coordinates of the mouse click.
            \param camera The camera of the game.
        */
        public static Button getMenuButtonClicked(Vector2 mouseCoordinates, Camera camera)
        {
            Unit unit = GameState.selectedUnit;
            Button[] menuButtons = unit.getButtons();
            int clickX = (int)mouseCoordinates.X;   // coordinates relative to map
            int clickY = (int)mouseCoordinates.Y;
            int stationaryClickX = (int)mouseCoordinates.X + (int)camera.Position.X;    // coordinates relative to window
            int stationaryClickY = (int)mouseCoordinates.Y + (int)camera.Position.Y;

            for (int i = 0; i < menuButtons.Length; i++)
            {
                int buttonX = (int)menuButtons[i].getPixelCoordinates().X;
                int buttonY = (int)menuButtons[i].getPixelCoordinates().Y;
                if (buttonX <= clickX && clickX < buttonX + 128 && buttonY <= clickY && clickY < buttonY + 32)
                {
                    if (GameState.dropDownMenuOpen)
                    {
                        return menuButtons[i];
                    }
                }
            }

            int ButtonX = (int)unit.getButtonType(ButtonType.AttackConfirm).getPixelCoordinates().X;
            int ButtonY = (int)unit.getButtonType(ButtonType.AttackConfirm).getPixelCoordinates().Y;
            if (ButtonX + 90 <= stationaryClickX && stationaryClickX < ButtonX + 214 &&
                ButtonY + 127 <= stationaryClickY && stationaryClickY < ButtonY + 172)
            {
                if (GameState.attackConfirmOpen)
                {
                    return unit.getButtonType(ButtonType.AttackConfirm);
                }
            }
            return null;
        }

        /**
        Enables scrolling of the map based on mouse position.
            \param camera The camera of the game.
            \param mouseX x-coordinate of the mouse position.
            \param mouseY y-coordinate of the mouse position.
        */
        public static void scrollMap(Camera camera, int mouseX, int mouseY)
        {
            const int scrollSpeed = 3;      // change in pixels while scrolling
            const int boundaryX = -640;
            const int boundaryY = -320;

            if (Game.Instance.IsActive)     // if game window is in focus
            {
                // scroll map to the left
                if (mouseX <= 0 && camera.Position.X < 0)
                {
                    camera.Position = new Vector2(camera.Position.X + scrollSpeed, camera.Position.Y);
                }

                // scroll map to the right
                if (mouseX >= GameState.SCREEN_WIDTH && camera.Position.X > boundaryX)
                {
                    camera.Position = new Vector2(camera.Position.X - scrollSpeed, camera.Position.Y);
                }

                // scroll map downwards
                if (mouseY >= GameState.SCREEN_HEIGHT && camera.Position.Y > boundaryY)
                {
                    camera.Position = new Vector2(camera.Position.X, camera.Position.Y - scrollSpeed);
                }

                // scroll map upwards
                if (mouseY <= 0 && camera.Position.Y < 0)
                {
                    camera.Position = new Vector2(camera.Position.X, camera.Position.Y + scrollSpeed);
                }
            }
        }
    }
}