﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

using View;
using System.Diagnostics;

namespace Model
{
    /// <summary>
    /// The Warrior model class, extends Unit
    /// This unit excels in dealing melee physical damage, with a high defense, but suffers from lack of any magical attack or resistance
    /// </summary>
    class Warrior : Unit
    {
        /**
        Sets and returns whether or not unit is alive
        */
        public bool Alive { get; set; }
        /**
        Sets and returns a unit's Speed
        */
        public int Speed { get; set; }
        /**
        Sets and returns a unit's Defense
        */
        public int Def { get; set; }
        /**
        Sets and returns a unit's Resistance
        */
        public int Res { get; set; }
        /**
        Sets and returns a unit's Level
        */
        public int Level { get; set; }
        /**
        returns weapon the unit is currently equipping
        */
        public Weapon equippedWeapon { get; set; }
        /**
        the current frame the sprite is on 
        */
        public int currentFrame { get; set; }

        private readonly int movability = 5;        // Archer movement is permanently set to 5
        private Weapon[] equipableWeapons;          // Array of all equipable weapons
        private int str;                            // unit strength
        private int intelligence;                   // unit intelliegence
        private int skill;                          // unit skill
        private int hp;                             // unit hp
        private Texture2D spriteImage;              // the char sprite
        private Vector2 pixelCoordinates;           // the pixel coordinate of the unit
        private Tuple<int, int> position;           // the tile location of the unit
        private Button[] buttons = new Button[9];   // the buttons associated with the unit
        private Texture2D charInfo, charAttackInfo; // The character and attack info textures

        /**
        * The constructor for Unit Warrior
        * @param spriteImage The character sprite
        * @param attackButton The dropdownmenu attack button texture
        * @param moveButton The dropdownmenu move button texture
        * @param itemButton The dropdownmenu item button texture
        * @param waitButton The dropdownmenu wait button texture
        * @param confirmButton The attack confirm button texture
        * @param charInfo The character info popup texture
        * @param charAttackInfo The character attack menu popup texture
        * @param coordinates The unit's current coordinate on screen
        * @param player The player of which the unit belongs to
        */
        public Warrior(Texture2D spriteImage, Button[] unitButtons, Texture2D charInfo,
            Texture2D charAttackInfo, Vector2 coordinates)
        {
            this.spriteImage = spriteImage;
            buttons[0] = unitButtons[0];
            buttons[1] = unitButtons[1];
            buttons[2] = unitButtons[2];
            buttons[3] = unitButtons[3];
            buttons[4] = unitButtons[4];
            buttons[5] = unitButtons[5];
            buttons[6] = unitButtons[6];
            buttons[7] = unitButtons[7];
            buttons[8] = unitButtons[8];
            this.charInfo = charInfo;
            this.charAttackInfo = charAttackInfo;
            pixelCoordinates = coordinates;
            int positionX = (int)Math.Round(coordinates.X / 32);
            int positionY = (int)Math.Round(coordinates.Y / 32);
            position = new Tuple<int, int>(positionX, positionY);
            currentFrame = 1;
            setButtonCoordinates(pixelCoordinates);
            setInitialStats(); //sets initial warrior stats
        }

        //sets initial unit stats
        public void setInitialStats()
        {
            Alive = true;
            Level = 1;
            Hp = 10;
            Str = 5;
            Int = 1;
            Skill = 6;
            Speed = 8;
            Def = 8;
            Res = 4;
        }

        /**
        Sets the new strength value
        \n Gets the effective strength -> Unit strength + weapon strength
        */
        public int Str
        {
            get
            {
                return str + equippedWeapon.modStr;
            }
            set
            {
                str = value;
            }
        }

        /**
        Sets the new intelligence value
        \n Gets the effective intelligence -> Unit intelligence + weapon intelligence
        */
        public int Int
        {
            get
            {
                return intelligence + equippedWeapon.modInt;
            }
            set
            {
                intelligence = value;
            }
        }

        /**
        Sets the new skill value
        \n Gets the effective skill -> Unit skill + weapon skill
        */
        public int Skill
        {
            get
            {
                return skill + equippedWeapon.modSkill;
            }
            set
            {
                skill = value;
            }
        }

        /**
        Sets the hp of the unit.
        \n Gets the unit's hp.
        */
        public int Hp
        {
            get
            {
                return hp;
            }
            set
            {
                if (value <= 0)
                {
                    this.Alive = false;
                }

                hp = value;
            }
        }

        /**
        Returns the unit's movability range on grid (number of spaces the unit can move in one turn)
        */
        public int getMovability()
        {
            return movability;
        }

        /**
        returns all stats as an array
        */
        public int[] getStats()
        {
            int[] allStats = new int[7];
            allStats[0] = Level;
            allStats[1] = Str;
            allStats[2] = Int;
            allStats[3] = Skill;
            allStats[4] = Speed;
            allStats[5] = Def;
            allStats[6] = Res;
            return allStats;
        }

        /**
        returns weapons the unit can equip (TODO)
        */
        public Weapon[] getEquipableWeapons()
        {
            return equipableWeapons;
        }

        /**
        returns unit's class (warrior, mage, archer)
        */
        public UnitType getClass()
        {
            return UnitType.Warrior;
        }

        /**
        returns the sprite image of the unit
        */
        public Texture2D getSpriteImage()
        {
            return spriteImage;
        }

        /**
        returns the button texture at index i
        */
        public Texture2D getButtonImage(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack: // if attack clicked
                    return buttons[0].getImage();
                case ButtonType.Move: // if moved is clicked
                    return buttons[1].getImage();
                case ButtonType.Items: // if item is clicked
                    return buttons[2].getImage();
                case ButtonType.Wait: // if wait is clicked
                    return buttons[3].getImage();
                case ButtonType.AttackConfirm: // if attack confirm is clicked
                    return buttons[4].getImage();
                case ButtonType.Inventory1: // if item1 clicked
                    return buttons[5].getImage();
                case ButtonType.Inventory2: // if item2 is clicked
                    return buttons[6].getImage();
                case ButtonType.Inventory3: // if item3 is clicked
                    return buttons[7].getImage();
                case ButtonType.Inventory4: // if item4 is clicked
                    return buttons[8].getImage();
                default:
                    return null;
            }
        }

        /**
        indicates whether a button has already been previously selected or not
        */
        public bool isButtonActive(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack:
                    return buttons[0].Active;
                case ButtonType.Move:
                    return buttons[1].Active;
                case ButtonType.Items:
                    return buttons[2].Active;
                case ButtonType.Wait:
                    return buttons[3].Active;
                case ButtonType.AttackConfirm:
                    return buttons[4].Active;
                case ButtonType.Inventory1:
                    return buttons[5].Active;
                case ButtonType.Inventory2:
                    return buttons[6].Active;
                case ButtonType.Inventory3:
                    return buttons[7].Active;
                case ButtonType.Inventory4:
                    return buttons[8].Active;
                default:
                    return false;
            }
        }

        /**
        returns the char info screen texture
        */
        public Texture2D getCharInfo()
        {
            return charInfo;
        }

        /**
        returns the char attack info screen texture
        */
        public Texture2D getCharAttackInfo()
        {
            return charAttackInfo;
        }

        /**
        gets and sets unit's position by tile
        */
        public Tuple<int, int> Position
        {
            get
            {
                return position;
            }
            set
            {
                position = value;
                pixelCoordinates = new Vector2(value.Item1 * 32, value.Item2 * 32);
            }
        }

        /**
        returns the pixel coordinate of the unit
        \n sets the pixel coordinate, and also sets Position (which is the tile location of that coordinate)
        */
        public Vector2 PixelCoordinates
        {
            get
            {
                return pixelCoordinates;
            }
            set
            {
                pixelCoordinates = value;
                int positionX = (int)Math.Round(PixelCoordinates.X / 32);
                int positionY = (int)Math.Round(PixelCoordinates.Y / 32);
                position = new Tuple<int, int>(positionX, positionY);
            }
        }

        /**
        returns the dropdown menu buttons of the unit
        */
        public Button[] getButtons()
        {
            return buttons;
        }

        /**
        Returns the button type
        * @param buttonType The button to return (Move, Attack, Item, Wait, and attack confirm)
        */
        public Button getButtonType(ButtonType buttonType)
        {
            switch (buttonType)
            {
                case ButtonType.Attack:
                    return buttons[0];
                case ButtonType.Move:
                    return buttons[1];
                case ButtonType.Items:
                    return buttons[2];
                case ButtonType.Wait:
                    return buttons[3];
                case ButtonType.AttackConfirm:
                    return buttons[4];
                case ButtonType.Inventory1:
                    return buttons[5];
                case ButtonType.Inventory2:
                    return buttons[6];
                case ButtonType.Inventory3:
                    return buttons[7];
                case ButtonType.Inventory4:
                    return buttons[8];
                default:
                    return null;
            }
        }

        /**
        sets the coordinates of menu buttons
        * @param pixelCoordinates The pixel coordinate of the button
        */
        public void setButtonCoordinates(Vector2 pixelCoordinates)
        {
            int offsetInactive = 0;
            for (int i = 0; i < 4; i++)
            {
                if (!buttons[i].Active)
                {
                    offsetInactive -= 32;
                }
                buttons[i].setPixelCoordinates((int)(pixelCoordinates.X + 32), (int)(pixelCoordinates.Y + (i * 32) + offsetInactive));
            }
            buttons[4].setPixelCoordinates(330, 120);
            for (int i = 5; i < 9; i++)
            {
                buttons[i].setPixelCoordinates((int)(pixelCoordinates.X + 160), (int)(pixelCoordinates.Y + (i * 32 - 96)));
            }
        }

        /**
        returns the current sprite frame in animation sequence
        */
        public Rectangle getCurrentFrame() //return current frame the sprite is on
        {
            Rectangle screenBounds = new Rectangle(currentFrame * 32, 0, 32, 32);
            return screenBounds;
        }
    }
}
